// -*- C++ -*-
/*!
 * @file  RTC_String_Logger.cpp
 * @brief string Log save
 * @date $Date$
 *
 * $Id$
 */

#include "RTC_String_Logger.h"

#include <string.h>
#include <fstream>
using namespace std;
ofstream Logout_S( "String_LOG.csv", ios::trunc );
// Module specification
// <rtc-template block="module_spec">

double t0=0,t1=0,dt=0,sum_t=0;
static const char* rtc_string_logger_spec[] =
  {
    "implementation_id", "RTC_String_Logger",
    "type_name",         "RTC_String_Logger",
    "description",       "string Log save",
    "version",           "1.0.0",
    "vendor",            "AIZUK",
    "category",          "Logge",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_String_Logger::RTC_String_Logger(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_stringIn("String", m_string)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_String_Logger::~RTC_String_Logger()
{
}



RTC::ReturnCode_t RTC_String_Logger::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("String", m_stringIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>
  
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onFinalize()
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onExecute(RTC::UniqueId ec_id)
{
	struct timeval ti;//時間構造体


	if( m_stringIn.isNew() ){
		std::string s;//受信ｓｔｒｉｎｇ
		m_stringIn.read();//ポート読み込み
		s=m_string.data;//受信データ→受信ｓｔｒｉｎｇへ
		
		gettimeofday(&ti, NULL);//現在時刻取得　Ｓｔｒｉｎｇデータ取得時間
		t0=(double)(ti.tv_sec)+(double)(ti.tv_usec)*0.001*0.001;//μ秒→秒変換
		
		dt=t0-t1;//⊿ｔ
		if(t1==0)dt=0;//最初のループなら⊿ｔは０
		sum_t += dt;//⊿ｔ足しあわせ　ＲＴＣ起動からの経過時間へ
		
		Logout_S<<sum_t<<"\t"<< s << endl;//タイムスタンプ（ｓｕｍ_t）　￥ｔ　 受信データ　￥ｎ　で保存
		t1=t0;//⊿ｔ計算用　前回データ取得時間
	}
  
  
  
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_String_Logger::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



extern "C"
{
 
  void RTC_String_LoggerInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_string_logger_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_String_Logger>,
                             RTC::Delete<RTC_String_Logger>);
  }
  
};


